# This file was auto-generated by Fern from our API Definition.

import typing

from .trade_traversal_path import TradeTraversalPath
from .trade_traversal_path_segment import TradeTraversalPathSegment

TradeTraversalPathOrSegment = typing.Union[
    typing.List[TradeTraversalPath], typing.List[typing.List[TradeTraversalPathSegment]]
]
